## MTNetflixQueue
## Original Copyright Benjamin Trott 2002. Released under the Artistic License.
## Tweaked to Netflix2 inelegantly 9_15_03 for new Netflix format with added tags, by
## Oscar F Hills <oscar@ofhills.com> Updated again to part of Netflix Suite 9_24_03
## Updated 6/28/04 adding "position" tag, thanks to Jay Levitt for that idea - especially
## useful if you are displaying a random sampling from your queue.
## Current update: 5/19/05

=head1 NAME

MTNetflixQueue - Display and format your Netflix Queue

=head1 USAGE

=over 4

=item 1. Drop netflix-queue.pl into your plugins directory.

=item 2. Get your Netflix session ID from your browser cookies.

Log in to Netflix through your browser.

Look for the value of the NetflixShopperId cookie in your
browser preferences, after logging in. You'll use that below
where it says <your session id>

=item 3. Use the <MTNetflixQueue> tags to display your queue.

For example:

    <MTNetflixQueue session_id="<your session ID>">
    <a href="<$MTNetflixQueueURL$>"><$MTNetflixQueueTitle$></a><br />
    </MTNetflixQueue>

Available tags are:

    <MTNetflixQueue>
    <$MTNetflixQueueTitle$>
    <$MTNetflixQueueURL$>
    <$MTNetflixQueueImage$>
    <$MTNetflixQueueGenre$>
    <$MTNetflixQueueRating$>
    <$MTNetflixQueueAvailability$>
    <$MTNetflixQueuePosition$>
    
    Additional arguments for <MTNetfilxQueue> are "lastn" "firstn" and "randomn"
    These treat your queue as such, i.e., FIFO, but they allude to weblogs where
    the "lastn" are the "n" most recent posts. In this plugin , "lastn" returns the
    list of "n" movies at the top of your queue, and "firstn" returns the list of
    "n" movies at the bottom. "randomn" returns "n" random movies selected from your
    queue. So, <MTNetflixQueue session_id = "<your session ID>" lastn = "10"> will 
    return the next 10 movies you will be viewing. Caution: Use of the image tag uses THEIR bandwith.

=cut

package MT::Plugin::NetflixQueue;
use strict;
use MT::Template::Context;
use LWP::UserAgent;
use HTTP::Cookies;

MT::Template::Context->add_container_tag('NetflixQueue', \&queue);
MT::Template::Context->add_tag('NetflixQueueURL', \&queue_info);
MT::Template::Context->add_tag('NetflixQueueImage', \&queue_info);
MT::Template::Context->add_tag('NetflixQueueTitle', \&queue_info);
MT::Template::Context->add_tag('NetflixQueueAvailability', \&queue_info);
MT::Template::Context->add_tag('NetflixQueueRating', \&queue_info);
MT::Template::Context->add_tag('NetflixQueueGenre', \&queue_info);
MT::Template::Context->add_tag('NetflixQueuePosition', \&queue_info);

my $queue = 'http://www.netflix.com/Queue';

sub queue {
    my($ctx, $args) = @_;
    my $session_id = $args->{session_id};
    my $lastn = defined($args->{lastn})?$args->{lastn}:0;
    my $firstn = defined($args->{firstn})?$args->{firstn}:0;
    my $randomn = defined($args->{randomn})?$args->{randomn}:0;
    
    my $seed = srand(time());
    if (($lastn > 0 && $firstn > 0) || ($lastn > 0 && $randomn > 0) || ($firstn > 0 && $randomn > 0)) {
     return $ctx->error('Only one "n" parameter allowed');}
    my $noparms = 0;
    if (!$lastn && !$firstn && !$randomn) {
        $noparms = 1;
      }
    
    my $jar = HTTP::Cookies->new;
    $jar->set_cookie(0, 'NetflixShopperId', $session_id, '/', '.netflix.com');
    my $req = HTTP::Request->new(GET => $queue);
    $jar->add_cookie_header($req);
    my $ua = LWP::UserAgent->new;
    $ua->agent('NetflixWebServices/1.0');
    my $res = $ua->request($req);
    return $ctx->error("HTTP error: " . $res->status_line)
        unless $res->is_success;

    my $builder = $ctx->stash('builder');
    my $tokens = $ctx->stash('tokens');

    my $c = $res->content;
    my $out = '';
    my $count = 0;
    $c =~ m!<div class="dvd-queue">(.*?)</body>!sg;
    $c = $1;
    while ($c =~ m!s=bd>(.*?)</tr>!sg) {
      ++$count
      }
    my $total = $count;
    
   my $randcount = 0;
   my  (@randrec, @randq);
   if ($randomn) {       
       for (my $i = 0; $i < $count; $i++) {
          $randrec[$i] = $i+1;
          }
    shuffle (\@randrec);
      for (my $i = 0; $i < $randomn; $i++) {
          if ($i < $count) {
          $randq[$i] = $randrec[$i];}
       }
     @randq = sort {$b <=> $a} @randq;
     }
    


    while ($c =~ m!s=bd>(.*?)</tr>!sg) {
        my $rec = $1;
        # my @rec = $rec =~ m!eid=(.*?)">!sg;
        my @rec = $rec =~ m!eid=(.*?)&trk!sg;
        my $mid = $1;
        
        $mid =~ s!&lnkctr=RecListStar!!sg;
        $rec[6] = "http://a408.g.akamai.net/f/408/1284/24h/image.netflix.com/NetFlix_Assets/boxshots/small/".$mid.".jpg";
        my $url = "http://www.netflix.com/MovieDisplay?"."movieid=".$mid;
        $rec =~ m!\d\d\d\d">(.*?)</a>.*?s=ec>(.*?)</td>.*?s=e>(.*?)&nbsp\;</td>.*?s=sac>(.*?)</div>!sg;
        
        $rec[2] = $1;        
        $rec[3] = $2;
        $rec[4] = $3;
        $rec[5] = $4;
       
        if ($lastn || $noparms || ($firstn && ($count <= $firstn)) || ($randomn && (($count) == $randq[$randcount]) && ($randcount < $randomn))) {
             my $entry = { title => $rec[2], url => $url, availability => $rec[5],
                           rating => $rec[3], genre => $rec[4], image => $rec[6], position => ($total - $count + 1) };
             $ctx->stash('netflix_queue_entry', $entry);
             defined(my $o = $builder->build($ctx, $tokens))
                 or return $ctx->error($builder->errstr);
             $out .= $o;
             if ($lastn){--$lastn;}
             if ($randomn  && ($randcount < $randomn-1)){++$randcount;}
         }
         --$count;
     }
     $out;
     
sub shuffle {
    my $array = shift;
    my $i; 
    for ($i = @$array; --$i; ) {
       my $j = int rand ($i+1);
       next if $i == $j;
       @$array[$i,$j] = @$array[$j,$i];
        }
        } 

}

sub queue_info {
    my $n = $_[0]->stash('netflix_queue_entry');
    (my $tag = lc $_[0]->stash('tag')) =~ s!^netflixqueue!!;
    $n->{$tag};
}
     

1;

